package com.example.wtf.controller;

import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Commentary;
import com.example.wtf.repository.CommentaryRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/commentaries")
public class CommentaryController {
    @Autowired
    private CommentaryRepository repository;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<Commentary>> getCommentaries() {
        List<EntityModel<Commentary>> commentaries = new ArrayList<>();

        for (Commentary commentary : repository.findAll()) {
            EntityModel<Commentary> commentaryResource = EntityModel.of(
                    commentary,
                    linkTo(methodOn(CommentaryController.class).getCommentary(commentary.getId())).withSelfRel(),
                    linkTo(methodOn(CommentaryController.class).deleteCommentary(commentary.getId())).withRel("Delete Commentary")
            );
            commentaries.add(commentaryResource);
        }

        return CollectionModel.of(
                commentaries,
                linkTo(methodOn(CommentaryController.class).addCommentary(new Commentary())).withRel("Add Commentary")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<Commentary> getCommentary(@PathVariable Long id) {
        Commentary commentary = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Commentary not found"));

        return EntityModel.of(
                commentary,
                linkTo(methodOn(CommentaryController.class).deleteCommentary(commentary.getId())).withRel("Delete Commentary"),
                linkTo(methodOn(CommentaryController.class).getCommentaries()).withRel("Commentaries")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<Commentary> addCommentary(@RequestBody Commentary commentary) {
        repository.save(commentary);
        return EntityModel.of(
                commentary,
                linkTo(methodOn(CommentaryController.class).getCommentary(commentary.getId())).withSelfRel(),
                linkTo(methodOn(CommentaryController.class).deleteCommentary(commentary.getId())).withRel("Delete Commentary"),
                linkTo(methodOn(CommentaryController.class).getCommentaries()).withRel("Commentaries")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<Commentary> updateCommentary(@PathVariable Long id, @RequestBody Commentary update) {
        Commentary commentary = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Commentary not found"));

        if (update.getUser() != null) {
            commentary.setUser(update.getUser());
        }
        if (update.getQuote() != null) {
            commentary.setQuote(update.getQuote());
        }
        if (update.getComment() != null && !update.getComment().isEmpty()) {
            commentary.setComment(update.getComment());
        }
        if (update.getDateC() != null) {
            commentary.setDateC(update.getDateC());
        }

        repository.save(commentary);

        return EntityModel.of(
                commentary,
                linkTo(methodOn(CommentaryController.class).updateCommentary(id, new Commentary())).withSelfRel()
        );
    }

    @DeleteMapping("/delete/{id}")
    public @ResponseBody EntityModel<Commentary> deleteCommentary(@PathVariable Long id) {
        Commentary commentary = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Commentary not found"));

        repository.delete(commentary);

        return EntityModel.of(
                commentary,
                linkTo(methodOn(CommentaryController.class).getCommentaries()).withRel("Commentaries")
        );
    }
}
