package com.example.wtf.controller;

import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Favorite;
import com.example.wtf.repository.FavoriteRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/favorites")
public class FavoriteController {
    @Autowired
    private FavoriteRepository repository;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<Favorite>> getFavorites() {
        List<EntityModel<Favorite>> favorites = new ArrayList<>();

        for (Favorite favorite : repository.findAll()) {
            EntityModel<Favorite> favoriteResource = EntityModel.of(
                    favorite,
                    linkTo(methodOn(FavoriteController.class).getFavorite(favorite.getId())).withSelfRel(),
                    linkTo(methodOn(FavoriteController.class).deleteFavorite(favorite.getId())).withRel("Delete Favorite")
            );
            favorites.add(favoriteResource);
        }

        return CollectionModel.of(
                favorites,
                linkTo(methodOn(FavoriteController.class).addFavorite(new Favorite())).withRel("Add Favorite")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<Favorite> getFavorite(@PathVariable Long id) {
        Favorite favorite = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Favorite not found"));

        return EntityModel.of(
                favorite,
                linkTo(methodOn(FavoriteController.class).deleteFavorite(favorite.getId())).withRel("Delete Favorite"),
                linkTo(methodOn(FavoriteController.class).getFavorites()).withRel("Favorites")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<Favorite> addFavorite(@RequestBody Favorite favorite) {
        repository.save(favorite);
        return EntityModel.of(
                favorite,
                linkTo(methodOn(FavoriteController.class).getFavorite(favorite.getId())).withSelfRel(),
                linkTo(methodOn(FavoriteController.class).deleteFavorite(favorite.getId())).withRel("Delete Favorite"),
                linkTo(methodOn(FavoriteController.class).getFavorites()).withRel("Favorites")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<Favorite> updateFavorite(@PathVariable Long id, @RequestBody Favorite update) {
        Favorite favorite = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Favorite not found"));

        if (update.getUser() != null) {
            favorite.setUser(update.getUser());
        }
        if (update.getQuote() != null) {
            favorite.setQuote(update.getQuote());
        }

        repository.save(favorite);

        return EntityModel.of(
                favorite,
                linkTo(methodOn(FavoriteController.class).updateFavorite(id, new Favorite())).withSelfRel()
        );
    }

    @DeleteMapping("/delete/{id}")
    public @ResponseBody EntityModel<Favorite> deleteFavorite(@PathVariable Long id) {
        Favorite favorite = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Favorite not found"));

        repository.delete(favorite);

        return EntityModel.of(
                favorite,
                linkTo(methodOn(FavoriteController.class).getFavorites()).withRel("Favorites")
        );
    }
}
