package com.example.wtf.controller;

import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Quiz;
import com.example.wtf.repository.QuizRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/quizzes")
public class QuizController {

    @Autowired
    private QuizRepository repository;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<Quiz>> getQuizzes() {
        List<EntityModel<Quiz>> quizzes = new ArrayList<>();

        for (Quiz quiz : repository.findAll()) {
            EntityModel<Quiz> quizResource = EntityModel.of(
                    quiz,
                    linkTo(methodOn(QuizController.class).getQuiz(quiz.getId())).withSelfRel(),
                    linkTo(methodOn(QuizController.class).deleteQuiz(quiz.getId())).withRel("Delete Quiz")
            );
            quizzes.add(quizResource);
        }

        return CollectionModel.of(
                quizzes,
                linkTo(methodOn(QuizController.class).addQuiz(new Quiz())).withRel("Add Quiz")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<Quiz> getQuiz(@PathVariable Long id) {
        Quiz quiz = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Quiz not found"));

        return EntityModel.of(
                quiz,
                linkTo(methodOn(QuizController.class).deleteQuiz(quiz.getId())).withRel("Delete Quiz"),
                linkTo(methodOn(QuizController.class).getQuizzes()).withRel("Quizzes")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<Quiz> addQuiz(@RequestBody Quiz quiz) {
        repository.save(quiz);
        return EntityModel.of(
                quiz,
                linkTo(methodOn(QuizController.class).getQuiz(quiz.getId())).withSelfRel(),
                linkTo(methodOn(QuizController.class).deleteQuiz(quiz.getId())).withRel("Delete Quiz"),
                linkTo(methodOn(QuizController.class).getQuizzes()).withRel("Quizzes")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<Quiz> updateQuiz(@PathVariable Long id, @RequestBody Quiz update) {
        Quiz quiz = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Quiz not found"));

        if (update.getTitle() != null && !update.getTitle().isEmpty()) {
            quiz.setTitle(update.getTitle());
        }
        if (update.getNbQuest() != null) {
            quiz.setNbQuest(update.getNbQuest());
        }
        if (update.getImage() != null) {
            quiz.setImage(update.getImage());
        }

        repository.save(quiz);

        return EntityModel.of(
                quiz,
                linkTo(methodOn(QuizController.class).updateQuiz(id, new Quiz())).withSelfRel()
        );
    }

    @DeleteMapping("/delete/{id}")
    public @ResponseBody EntityModel<Quiz> deleteQuiz(@PathVariable Long id) {
        Quiz quiz = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Quiz not found"));

        repository.delete(quiz);

        return EntityModel.of(
                quiz,
                linkTo(methodOn(QuizController.class).getQuizzes()).withRel("Quizzes")
        );
    }
}
