package com.example.wtf.controller;

import com.example.wtf.exception.ResourceForbidden;
import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Quote;
import com.example.wtf.repository.QuoteRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/quotes")
public class QuoteController {

    @Autowired
    private QuoteRepository repository;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<Quote>> getQuotes() {
        List<EntityModel<Quote>> quotes = new ArrayList<>();

        for (Quote quote : repository.findAll()) {
            EntityModel<Quote> quoteResource = EntityModel.of(
                    quote,
                    linkTo(methodOn(QuoteController.class).getQuote(quote.getId())).withSelfRel(),
                    linkTo(methodOn(QuoteController.class).updateQuote(quote.getId(), new Quote())).withRel("Update quote"),
                    linkTo(methodOn(QuoteController.class).deleteQuote(quote.getId())).withRel("Delete quote")
            );
            quotes.add(quoteResource);
        }

        return CollectionModel.of(
                quotes,
                linkTo(methodOn(QuoteController.class).addQuote(new Quote())).withRel("Add Quote")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<Quote> getQuote(@PathVariable Long id) {
        Quote quote = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Quote not found"));

        return EntityModel.of(
                quote,
                linkTo(methodOn(QuoteController.class).updateQuote(quote.getId(), new Quote())).withRel("Update quote"),
                linkTo(methodOn(QuoteController.class).deleteQuote(quote.getId())).withRel("Delete quote"),
                linkTo(methodOn(QuoteController.class).getQuotes()).withRel("Quotes")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<Quote> addQuote(@RequestBody Quote quote) {

        repository.save(quote);

        return EntityModel.of(
                quote,
                linkTo(methodOn(QuoteController.class).getQuote(quote.getId())).withSelfRel(),
                linkTo(methodOn(QuoteController.class).updateQuote(quote.getId(), new Quote())).withRel("Update quote"),
                linkTo(methodOn(QuoteController.class).deleteQuote(quote.getId())).withRel("Delete quote"),
                linkTo(methodOn(QuoteController.class).getQuotes()).withRel("Quotes")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<Quote> updateQuote(@PathVariable Long id,
                                                        @RequestBody Quote update) {
        Quote quote = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Quote not found"));

        if (update.getContent() != null && !update.getContent().isEmpty()) {
            quote.setContent(update.getContent());
        }
        if (update.getLikes() != null) {
            quote.setLikes(update.getLikes());
        }
        if (update.getLanguage() != null && !update.getLanguage().isEmpty()) {
            quote.setLanguage(update.getLanguage());
        }
        if (update.getIsValid() != null) {
            quote.setIsValid(update.getIsValid());
        }
        if (update.getReason() != null && !update.getReason().isEmpty()) {
            quote.setReason(update.getReason());
        }

        repository.save(quote);

        return EntityModel.of(
                quote,
                linkTo(methodOn(QuoteController.class).getQuote(id)).withSelfRel(),
                linkTo(methodOn(QuoteController.class).deleteQuote(id)).withRel("Delete quote"),
                linkTo(methodOn(QuoteController.class).getQuotes()).withRel("Quotes")
        );
    }

    @DeleteMapping("/delete/{id}")
    public @ResponseBody EntityModel<Quote> deleteQuote(@PathVariable Long id) {
        Quote quote = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Quote not found"));

        repository.delete(quote);

        return EntityModel.of(
                quote,
                linkTo(methodOn(QuoteController.class).getQuotes()).withRel("Quotes")
        );
    }
}